@@ -14,6 +14,7 @@
 #![warn(unstable_features)]
 
 mod client;
+pub(crate) mod py_async;
 mod py_err;
 mod server;
 
@@ -25,16 +26,6 @@ use tracing_subscriber::layer::SubscriberExt;
 use tracing_subscriber::util::SubscriberInitExt;
 use tracing_subscriber::{EnvFilter, fmt};
 
-macro_rules! inherit_doc {
-    (#[inherit_doc = $y:literal] $x:item) => {
-        #[cfg_attr(feature = "external-docs", doc =
-                                include_str!(concat!(env!("PERSPECTIVE_CLIENT_DOCS_PATH"), $y)))]
-        $x
-    };
-}
-
-pub(crate) use inherit_doc;
-
 /// Create a tracing filter which mimics the default behavior of reading from
 /// env, customized to exclude timestamp.
 /// [`tracing` filter docs](https://docs.rs/tracing-subscriber/latest/tracing_subscriber/layer/index.html#per-layer-filtering)
@@ -50,21 +41,33 @@ fn init_tracing() {
         .init();
 }
 
-/// A Python module implemented in Rust.
+#[pyfunction]
+fn num_cpus() -> i32 {
+    perspective_server::num_cpus()
+}
+
+#[pyfunction]
+fn set_num_cpus(num_cpus: i32) {
+    perspective_server::set_num_cpus(num_cpus)
+}
+
+/// Perspective Python main module.
 #[pymodule]
 fn perspective(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     init_tracing();
     m.add_class::<client::client_sync::Client>()?;
-    m.add_class::<server::PySyncServer>()?;
-    m.add_class::<server::PySyncSession>()?;
+    m.add_class::<server::PyServer>()?;
+    m.add_class::<server::PyAsyncServer>()?;
+    m.add_class::<server::PySession>()?;
+    m.add_class::<server::PyAsyncSession>()?;
     m.add_class::<client::client_sync::Table>()?;
     m.add_class::<client::client_sync::View>()?;
     m.add_class::<client::client_async::AsyncClient>()?;
     m.add_class::<client::client_async::AsyncTable>()?;
     m.add_class::<client::client_async::AsyncView>()?;
     m.add_class::<client::proxy_session::ProxySession>()?;
-
     m.add("PerspectiveError", py.get_type::<PyPerspectiveError>())?;
-
+    m.add_function(wrap_pyfunction!(num_cpus, m)?)?;
+    m.add_function(wrap_pyfunction!(set_num_cpus, m)?)?;
     Ok(())
 }
